/*****************************************************************************
FILE:               Oracle7Expression.m
LANGUAGE:           Objective-C
SYSTEM:             GNUstep
USER-INTERFACE:     None
DESCRIPTION

    This module defines the SQL expressions specific to the Oracle7 Adaptor.

AUTHORS
    <PJB> Pascal J. Bourguignon <pjb@imaginet.fr>
MODIFICATIONS
    1999/07/24 <PJB> Creation.
    2000/02/14 <PJB> Added methods to provide the Oracle7Channel the 
                     attributes used in the select.
BUGS
    - ...
LEGAL
    Written by:  Pascal J. Bourguignon <pjb@imaginet.fr>
    Funds from:  Orange Concept (http://www.orange-concept.com/)
    Date:        August 1999 - February 2000

    This file is part of the Oracle7 Adaptor, GNUstep Database Library.
    The Oracle7 Adaptor allows access to Oracle[678] databases from GDL.

    Copyright (C) 2000 Free Software Foundation, Inc.

    This  library is  free software;  you can  redistribute  it and/or
    modify it under the terms of the GNU Lesser General Public License
    as published by the Free  Software Foundation; either version 2 of
    the License, or (at your option) any later version.

    This library  is distributed in the  hope that it  will be useful,
    but  WITHOUT ANY WARRANTY;  without even  the implied  warranty of
    MERCHANTABILITY or FITNESS FOR  A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have  received a copy of the  GNU Lesser General Public
    License  along  with this  library;  if  not,  write to  the  Free
    Software  Foundation, Inc.,  59 Temple  Place, Suite  330, Boston,
    MA 02111-1307 USA
*****************************************************************************/

#include <Oracle7SQLExpression.h>
#include <Oracle7Channel-ForFriends.h>

@implementation Oracle7SQLExpression

// NSObject methods:

    // None overriden.


// EOSQLExpression methods:

    +(Class)selectExpressionClass
    {
        return(self);
    }//selectExpressionClass;


    +(Class)insertExpressionClass
    {
        return(self);
    }//insertExpressionClass;


    +(Class)deleteExpressionClass
    {
        return(self);
    }//deleteExpressionClass;


    +(Class)updateExpressionClass
    {
        return(self);
    }//updateExpressionClass;


    +(NSString*)selectExpressionForAttributes:(NSArray*)attributes 
                                         lock:(BOOL)lock
                                    qualifier:(EOQualifier*)qualifier
                                   fetchOrder:(NSArray*)fetchOrder
                                      channel:(EOAdaptorChannel*)channel
    {
        NSString* result;
        if([channel respondsToSelector:@selector(willSelectAttributes:)]){
            [(Oracle7Channel*)channel willSelectAttributes:attributes];
        }
        result=[super selectExpressionForAttributes:attributes 
                      lock:lock
                      qualifier:qualifier
                      fetchOrder:fetchOrder
                      channel:channel];
        return(result);
    }//selectExpressionForAttributes:lock:qualifier:fetchOrder:channel:;



    -(NSString*)lockClause
    {
        return(@" FOR UPDATE ");
    }//lockClause;


// Oracle7SQLExpression methods:

    // None.

@end // Oracle7SQLExpression;


/*** Oracle7SQLExpression.m           -- 2000-02-21 07:11:23 -- PJB ***/
